import javafx.geometry.Pos;

/**********************************************************************************************************************

 AUFGABENBLATT 6 - Allgemeine Informationen

 Das Projekt Aufgabenblatt6 besteht aus fünf Aufgaben in je einer Klasse, die Sie erweitern und in den Übungen vom 
 30.11. bis 4.12. präsentieren können müssen.

 Achten Sie bei der Implementierung der Klassen auf folgende Punkte:

 - Einige Aufgabenstellungen sind in eine Geschichte eingebettet und auf einer höheren Abstraktionsebene beschrieben
   als in bisherigen Übungen. Rechnen Sie daher mit einem höheren Aufwand zum Verstehen der Aufgaben.
 - Ihr Programm sollte kompilierbar und ausführbar sein.
 - Testen und kommentieren Sie Ihre Programme ausführlich.
 - Bei jeder Aufgabe finden Sie Zusatzfragen. Diese Zusatzfragen beziehen sich thematisch auf das erstellte Programm.
   Sie müssen diese Zusatzfragen in der Übung beantworten können.

 Abgabe: Die Abgabe erfolgt in TUWEL indem Sie bis Montag, den 30.11.2015 um 06:00 Ihre Kreuzerl des aktuellen
 Aufgabenblattes eintragen. Nur durch Ankreuzen können Sie auch Punkte bekommen. Allerdings müssen Sie
 angekreuzte Aufgaben auch vorzeigen können.

 ***********************************************************************************************************************/
/*
    Aufgabenstellung zur Klasse Position:
        Die letzten Wochen vor Weihnachten sind für Karlis Fahrradbotendienst die schwierigsten. Einerseits gibt es
        mehr Botenfahrten als im restlichen Jahr, andererseits sind die Straßenverhältnisse oft schlecht, und angeblich
        bleibt immer wieder einmal ein Fahrradbote an einem Punschstand hängen. Dadurch erreicht so manche Sendung den
        Empfänger nicht wie zugesichert innerhalb einer Stunde, wobei Fahrten nur zwischen 8:00 und 18:00 Uhr erfolgen.
        Um die Ursachen von Zustellproblemen leichter ermitteln zu können, werden die Radfahrer mit GPS-Trackern
        ausgestattet, die nach jeder Fahrt ausgewertet werden. Das Auslesen eines GPS-Trackers gibt ein Array von
        Objekten der hier implementierten Klasse 'Position' zurück. Für ca. jede Minute der Fahrt gibt es einen Eintrag
        im Array, chronologisch sortiert. Jeder Array-Eintrag enthält Informationen zur Uhrzeit (als Anzahl der seit
        Mitternacht vergangenen Sekunden) und der genauestmöglichen Position in Wien und Umgebung, dargestellt durch
        Longitude (also der geographischen Länge im Wertebereich von etwa 15.0 bis 17.0) und Latitude (geographischer
        Breite etwa zwischen 47.0 und 49.0). Methoden von 'Position' sollen bei der Auswertung der Daten helfen und
        sich den Kommentaren entsprechend verhalten.

    Zusatzfragen:
    1. Welche Teile der Aufgabenstellung beeinflussen welche Teile der Lösung auf welche Weise?
       Welche Teile der Aufgabenstellung haben dagegen keinen Einfluss auf die Lösung?
    2. Warum sollen Objektvariablen als private deklariert sein?
    3. Woran kann man einen Konstruktor erkennen?
    4. Warum ist isStopped als static definiert, isClose und isCloseToPath aber nicht?
    5. Warum erlauben die Vergleiche einen Abstand eps zwischen Standorten (statt einfacher Vergleiche mittels ==)?
*/
public class Position {

    // : Object variables shall be declared here.
    private int time;           // seconds since midnight, 28800 (8:00) <= value >= 64800
    private double longitude;   // 15.0 <= value >= 17.0
    private double latitude;    // 47.0 <= value >= 49.0



    // : A public constructor for Position shall be declared here, providing values for all object variables.


    // constructor without parameter
    public Position() {
        this.set(0, 0.0, 0.0);
    }

    // constructor with parameter
    public Position(int time, double longitude, double latitude) {
        this.set(time, longitude, latitude);
    }

    // setter
    private void set (int time, double longitude, double latitude) {
        this.time = time;
        this.longitude = longitude;
        this.latitude = latitude;
    }

    // getter
    public int getTime() {
        return this.time;
    }

    // Returns true if (and only if) longitude and latitude of this position (the current object) differ from
    // longitude and latitude of x by at most eps.
    public boolean isClose(Position x, double eps) {
        // : Implementation is your task.
        // https://de.wikipedia.org/wiki/Epsilontik
        return x != null &&
                Math.abs(x.longitude - this.longitude) <= eps &&
                Math.abs(x.latitude - this.latitude) <= eps;
    }

    // Returns true if (and only if) longitude and latitude of this position (the current object) differ from
    // longitude and latitude of any element in xs by at most eps.
    public boolean isCloseToPath(Position[] xs, double eps) {
        // : Implementation is your task.
        for (Position val : xs) {
            // test if val != null
            if (val != null && isClose(val, eps)) return true;
        }
        return false;
    }

    // Parameter xs contains the positions of a tour. A tour is regarded as being stopped at a position in the array
    // if this position is close (difference at most eps) to each of the following 4 positions in the array.
    // The method returns the time (in seconds since midnight) of the first stop (lowest index in the array),
    // or -1 if there is no stop.
    public static int isStopped(Position[] xs, double eps) {
        // TODO: Implementation is your task.
        //
        int interval = 4;

        for (int i = 0; i < xs.length; i++) {
            // check next interval positions, whether they are close (eps) or not
            Position[] temp = new Position[interval];
            // start at 1, because i+0 would compare with same position
            // also check if index out of bounds
            for (int j = 1; (j <= interval) && (i+j < xs.length); j++) {
                // j = 1, but first index is 0
                temp[j-1] = xs[i+j];
            }
            if (xs[i].isCloseToPath(temp, eps)) return xs[i].time;
        }
        return -1;
    }

    // Returns true if (and only if) the time of each position in xs is higher than that of each other position at a
    // lower index.
    public static boolean chronologic(Position[] xs) {
        // : Implementation is your task.
        int save = 0;

        for(Position val: xs) {
            if (val.time <= save) return false;
            save = val.time;
        }
        return true;
    }


    // Just for testing ...
    public static void main(String[] args) {
        // Den Rumpf dieser Methode können Sie beliebig verändern.
        Position[] test1 = {new Position(28800, 15.0, 47.0),
                new Position(28860, 16.0, 47.0),
                new Position(28920, 16.0, 47.0),
                new Position(28980, 16.0, 47.0),
                new Position(29040, 16.0, 47.0),
                new Position(29100, 16.0, 47.0)
                };
        Position[] test2 = {new Position(28800, 16.0, 47.0), new Position(28860, 17.0, 47.0)};
        Position[] test3 = {new Position(28800, 15.0, 47.0),
                new Position(28860, 16.0, 47.0),
                new Position(28920, 16.0, 47.0),
                new Position(28980, 16.0, 47.0),
                new Position(29040, 16.0, 47.0),
                new Position(28100, 16.0, 47.0)
        };
        System.out.println("isClose:");
        System.out.println(test1[0].isClose(test1[1],10.0) + " should be true");
        System.out.println(test1[0].isClose(test1[1],0.9) + " should be false");
        System.out.println("\nisCloseToPath:");
        System.out.println(test1[0].isCloseToPath(test2, 0.0) + " should be false");
        System.out.println(test1[0].isCloseToPath(test2, 1.0) + " should be true");
        System.out.println("\nisStopped:");
        System.out.println(isStopped(test1,1.0));

        // TODO: testen
        System.out.println(isStopped(test1,0.0));

        System.out.println("\nchronologic:");
        System.out.println(chronologic(test1) + " should be true");
        System.out.println(chronologic(test3) + " should be false");
    }
}
