import java.util.*;

/*
    Aufgabenstellung zur Klasse StreetMap:

    Um die Suche nach einer Zieladresse zu erleichtern, haben die Fahrer in Karlis Fahrradbotendienst einen einfachen
    elektronischen Stadtplan bei sich: Nach Eingabe einer Adresse erscheinen der Name des entsprehenden Distrikts und
    die GPS-Koordinaten. Objekte der Klasse StreetMap stellen einen wesentlichen Teil des Stadtplans dar.
    Vervollständigen Sie die Implementierung, so wie in den Kommentaren beschrieben.

    Bitte verwenden Sie in der Implementierung von StreetMap KEIN Array, sondern nur Objekte von Typen wie Queue, Deque
    und Map (neben MapData - siehe unten).

    Zusatzfragen:
    1. Wieso ist die zusätzliche Klasse MapData sinnvoll?
    2. Wie werden die Daten in MapData zugreifbar? Geht das auch, wenn die Objektvariablen private sind?
    3. Wofür ist Queue besser geeignet, wofür Map?
    4. Warum ist Double mit großem Anfangsbuchstaben geschrieben (nicht double)?
*/

public class StreetMap {

    // TODO: Object variables shall be declared here.
    private Map<String, MapData> map = new HashMap<>();



    // The parameters specify the initial contents of a street map:
    //     addr:  addresses that can be found in the street map
    //     distr: names of the districts of corresponding addresses
    //     lon:   longitudes in the GPS coordinates
    //     lat:   latitude in the GPS coordinates
    // All queues in the parameters are of the same size.<
    // All entries at the same position (1st, 2nd, 3rd, ...) belong together.
    public StreetMap(Queue<String> addr, Queue<String> distr, Queue<Double> lon, Queue<Double> lat) {
        // : Implementation is your task.

        // queues don't have the same size
        // if (addr.size() != distr.size() && addr.size() != lon.size() && addr.size() != lat.size()) {
        //     System.err.println("StreetMap Constructor failed - wrong size(s)");
        //     return;
        // }
        while (!addr.isEmpty())
            map.put(addr.poll(), new MapData(distr.poll(), lon.poll(), lat.poll()));
    }

    // Returns all data (district and GPS coordinates) for address addr.
    // Returns null if no data can be found for this address.
    public MapData find(String addr) {
        // : Implementation is your task.
        return !this.map.containsKey(addr) ? null : this.map.get(addr);
    }

    // Returns true if (and only if) address addr is in district distr.
    public boolean inDistrict(String addr, String distr) {
        // : Implementation is your task.
        return (this.find(addr) != null &&
                (this.find(addr).distr.equals(distr)));
    }

    // Adds a new address addr to the street map, where newData are the data to be associated with this address.
    // However, if the address already exists, the old data are replaced with newData.
    // true is returned if the address was replaced, false if a new address was added.
    public boolean newData(String addr, MapData newData) {
        // : Implementation is your task.
        // if (this.find(addr) == null) {
        //     this.map.put(addr, newData);
        //     return false;
        // } else {
        //     this.map.put(addr, newData);
        //     return true;
        // }
        return this.map.put(addr, newData) != null;
    }

    // To test the implementation several objects of StreetMap (each with several addresses) are created,
    // all methods are called, and data associated with found addresses are printed.
    public static void main(String[] args) {
        // TODO: Implementation is your task.

        // queues + data

        Queue<String> distr = new LinkedList<>();
        distr.offer("District 1");
        distr.offer("District 1");
        distr.offer("District 2");
        distr.offer("District 2");
        distr.offer("District 3");

        Queue<String> addr = new LinkedList<>();
        addr.offer("Street 1");
        addr.offer("Street 2");
        addr.offer("Street 3");
        addr.offer("Street 4");
        addr.offer("Street 5");

        Queue<Double> lon = new LinkedList<>();
        lon.offer(1.0);
        lon.offer(2.0);
        lon.offer(3.0);
        lon.offer(4.0);
        lon.offer(5.0);

        Queue<Double> lat = new LinkedList<>();
        lat.offer(5.0);
        lat.offer(4.0);
        lat.offer(3.0);
        lat.offer(2.0);
        lat.offer(1.0);

        // tests StreetMap and MapData Constructor
        StreetMap map1 = new StreetMap(addr,distr,lon,lat);

        // tests .inDistrict + .find
        System.out.println("map1.inDistrict(\"Street 1\", \"District 1\"): ");
        System.out.print(map1.inDistrict("Street 1", "District 1"));

        // tests .newData
        System.out.println(map1.newData("Street X", new MapData("District 1", 99.0, 99.0)));


        System.out.println();

    }
}

// Objects of MapData hold all data (district and GPS coordinates) associated with an address in the street map.
// It is necessary to get access to the data in objects of this type.
class MapData {
    // : Implementation is your task.
    public String distr;
    private double lon;
    private double lat;

    public MapData(String distr, double lon, double lat) {
        this.distr = distr;
        this.lon = lon;
        this.lat = lat;
    }
}
